@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -100,6 +100,8 @@ class CHost extends CHostGeneral {
 			'editable'					=> false,
 			'nopermissions'				=> null,
 			// filter
+			'evaltype'					=> TAG_EVAL_TYPE_AND_OR,
+			'tags'						=> null,
 			'filter'					=> null,
 			'search'					=> null,
 			'searchInventory'			=> null,
@@ -120,9 +122,10 @@ class CHost extends CHostGeneral {
 			'selectScreens'				=> null,
 			'selectInterfaces'			=> null,
 			'selectInventory'			=> null,
-			'selectHttpTests'           => null,
+			'selectHttpTests'			=> null,
 			'selectDiscoveryRule'		=> null,
 			'selectHostDiscovery'		=> null,
+			'selectTags'				=> null,
 			'countOutput'				=> false,
 			'groupCount'				=> false,
 			'preservekeys'				=> false,
@@ -382,6 +385,13 @@ class CHost extends CHostGeneral {
 					')';
 		}
 
+		// tags
+		if ($options['tags'] !== null && $options['tags']) {
+			$sqlParts['where'][] = CApiTagHelper::addWhereCondition($options['tags'], $options['evaltype'], 'h',
+				'host_tag', 'hostid'
+			);
+		}
+
 		// search
 		if (is_array($options['search'])) {
 			zbx_db_search('hosts h', $options, $sqlParts);
@@ -460,35 +470,40 @@ class CHost extends CHostGeneral {
 	/**
 	 * Add host.
 	 *
-	 * @param array  $hosts									An array with hosts data.
-	 * @param string $hosts[]['host']						Host technical name.
-	 * @param string $hosts[]['name']						Host visible name (optional).
-	 * @param array  $hosts[]['groups']						An array of host group objects with IDs that host will be added to.
-	 * @param int    $hosts[]['status']						Status of the host (optional).
-	 * @param array  $hosts[]['interfaces']					An array of host interfaces data.
-	 * @param int    $hosts[]['interfaces']['type']			Interface type.
-	 * @param int    $hosts[]['interfaces']['main']			Is this the default interface to use.
-	 * @param string $hosts[]['interfaces']['ip']			Interface IP (optional).
-	 * @param int    $hosts[]['interfaces']['port']			Interface port (optional).
-	 * @param int    $hosts[]['interfaces']['useip']		Interface shoud use IP (optional).
-	 * @param string $hosts[]['interfaces']['dns']			Interface shoud use DNS (optional).
-	 * @param int    $hosts[]['interfaces']['bulk']			Use bulk requests for interface (optional).
-	 * @param int    $hosts[]['proxy_hostid']				ID of the proxy that is used to monitor the host (optional).
-	 * @param int    $hosts[]['ipmi_authtype']				IPMI authentication type (optional).
-	 * @param int    $hosts[]['ipmi_privilege']				IPMI privilege (optional).
-	 * @param string $hosts[]['ipmi_username']				IPMI username (optional).
-	 * @param string $hosts[]['ipmi_password']				IPMI password (optional).
-	 * @param array  $hosts[]['inventory']					An array of host inventory data (optional).
-	 * @param array  $hosts[]['macros']						An array of host macros (optional).
-	 * @param string $hosts[]['macros'][]['macro']			Host macro (required if "macros" is set).
-	 * @param array  $hosts[]['templates']					An array of template objects with IDs that will be linked to host (optional).
-	 * @param string $hosts[]['templates'][]['templateid']	Template ID (required if "templates" is set).
-	 * @param string $hosts[]['tls_connect']				Connections to host (optional).
-	 * @param string $hosts[]['tls_accept']					Connections from host (optional).
-	 * @param string $hosts[]['tls_psk_identity']			PSK identity (required if "PSK" type is set).
-	 * @param string $hosts[]['tls_psk']					PSK (required if "PSK" type is set).
-	 * @param string $hosts[]['tls_issuer']					Certificate issuer (optional).
-	 * @param string $hosts[]['tls_subject']				Certificate subject (optional).
+	 * @param array  $hosts                                 An array with hosts data.
+	 * @param string $hosts[]['host']                       Host technical name.
+	 * @param string $hosts[]['name']                       Host visible name (optional).
+	 * @param array  $hosts[]['groups']                     An array of host group objects with IDs that host will be
+	 *                                                      added to.
+	 * @param int    $hosts[]['status']                     Status of the host (optional).
+	 * @param array  $hosts[]['interfaces']                 An array of host interfaces data.
+	 * @param int    $hosts[]['interfaces']['type']         Interface type.
+	 * @param int    $hosts[]['interfaces']['main']         Is this the default interface to use.
+	 * @param string $hosts[]['interfaces']['ip']           Interface IP (optional).
+	 * @param int    $hosts[]['interfaces']['port']         Interface port (optional).
+	 * @param int    $hosts[]['interfaces']['useip']        Interface shoud use IP (optional).
+	 * @param string $hosts[]['interfaces']['dns']          Interface shoud use DNS (optional).
+	 * @param int    $hosts[]['interfaces']['bulk']         Use bulk requests for interface (optional).
+	 * @param int    $hosts[]['proxy_hostid']               ID of the proxy that is used to monitor the host (optional).
+	 * @param int    $hosts[]['ipmi_authtype']              IPMI authentication type (optional).
+	 * @param int    $hosts[]['ipmi_privilege']             IPMI privilege (optional).
+	 * @param string $hosts[]['ipmi_username']              IPMI username (optional).
+	 * @param string $hosts[]['ipmi_password']              IPMI password (optional).
+	 * @param array  $hosts[]['tags']                       An array of tags (optional).
+	 * @param string $hosts[]['tags'][]['tag']              Tag name.
+	 * @param string $hosts[]['tags'][]['value']            Tag value.
+	 * @param array  $hosts[]['inventory']                  An array of host inventory data (optional).
+	 * @param array  $hosts[]['macros']                     An array of host macros (optional).
+	 * @param string $hosts[]['macros'][]['macro']          Host macro (required if "macros" is set).
+	 * @param array  $hosts[]['templates']                  An array of template objects with IDs that will be linked
+	 *                                                      to host (optional).
+	 * @param string $hosts[]['templates'][]['templateid']  Template ID (required if "templates" is set).
+	 * @param string $hosts[]['tls_connect']                Connections to host (optional).
+	 * @param string $hosts[]['tls_accept']                 Connections from host (optional).
+	 * @param string $hosts[]['tls_psk_identity']           PSK identity (required if "PSK" type is set).
+	 * @param string $hosts[]['tls_psk']                    PSK (required if "PSK" type is set).
+	 * @param string $hosts[]['tls_issuer']                 Certificate issuer (optional).
+	 * @param string $hosts[]['tls_subject']                Certificate subject (optional).
 	 *
 	 * @return array
 	 */
@@ -498,7 +513,7 @@ class CHost extends CHostGeneral {
 		$this->validateCreate($hosts);
 
 		$hostids = [];
-
+		$ins_tags = [];
 		foreach ($hosts as $host) {
 			// If visible name is not given or empty it should be set to host name.
 			if (!array_key_exists('name', $host) || !trim($host['name'])) {
@@ -520,6 +535,12 @@ class CHost extends CHostGeneral {
 			}
 			DB::insert('hosts_groups', $groupsToAdd);
 
+			if (array_key_exists('tags', $host)) {
+				foreach ($host['tags'] as $tag) {
+					$ins_tags[] = ['hostid' => $hostid] + $tag;
+				}
+			}
+
 			$options = [
 				'hosts' => $host
 			];
@@ -561,44 +582,51 @@ class CHost extends CHostGeneral {
 			}
 		}
 
+		if ($ins_tags) {
+			DB::insert('host_tag', $ins_tags);
+		}
+
 		return ['hostids' => $hostids];
 	}
 
 	/**
 	 * Update host.
 	 *
-	 * @param array  $hosts											An array with hosts data.
-	 * @param string $hosts[]['hostid']								Host ID.
-	 * @param string $hosts[]['host']								Host technical name (optional).
-	 * @param string $hosts[]['name']								Host visible name (optional).
-	 * @param array  $hosts[]['groups']								An array of host group objects with IDs that host will be replaced to.
-	 * @param int    $hosts[]['status']								Status of the host (optional).
-	 * @param array  $hosts[]['interfaces']							An array of host interfaces data to be replaced.
-	 * @param int    $hosts[]['interfaces']['type']					Interface type.
-	 * @param int    $hosts[]['interfaces']['main']					Is this the default interface to use.
-	 * @param string $hosts[]['interfaces']['ip']					Interface IP (optional).
-	 * @param int    $hosts[]['interfaces']['port']					Interface port (optional).
-	 * @param int    $hosts[]['interfaces']['useip']				Interface shoud use IP (optional).
-	 * @param string $hosts[]['interfaces']['dns']					Interface shoud use DNS (optional).
-	 * @param int    $hosts[]['interfaces']['bulk']					Use bulk requests for interface (optional).
-	 * @param int    $hosts[]['proxy_hostid']						ID of the proxy that is used to monitor the host (optional).
-	 * @param int    $hosts[]['ipmi_authtype']						IPMI authentication type (optional).
-	 * @param int    $hosts[]['ipmi_privilege']						IPMI privilege (optional).
-	 * @param string $hosts[]['ipmi_username']						IPMI username (optional).
-	 * @param string $hosts[]['ipmi_password']						IPMI password (optional).
-	 * @param array  $hosts[]['inventory']							An array of host inventory data (optional).
-	 * @param array  $hosts[]['macros']								An array of host macros (optional).
-	 * @param string $hosts[]['macros'][]['macro']					Host macro (required if "macros" is set).
-	 * @param array  $hosts[]['templates']							An array of template objects with IDs that will be linked to host (optional).
-	 * @param string $hosts[]['templates'][]['templateid']			Template ID (required if "templates" is set).
-	 * @param array  $hosts[]['templates_clear']					Templates to unlink and clear from the host (optional).
-	 * @param string $hosts[]['templates_clear'][]['templateid']	Template ID (required if "templates" is set).
-	 * @param string $hosts[]['tls_connect']						Connections to host (optional).
-	 * @param string $hosts[]['tls_accept']							Connections from host (optional).
-	 * @param string $hosts[]['tls_psk_identity']					PSK identity (required if "PSK" type is set).
-	 * @param string $hosts[]['tls_psk']							PSK (required if "PSK" type is set).
-	 * @param string $hosts[]['tls_issuer']							Certificate issuer (optional).
-	 * @param string $hosts[]['tls_subject']						Certificate subject (optional).
+	 * @param array  $hosts                                       An array with hosts data.
+	 * @param string $hosts[]['hostid']                           Host ID.
+	 * @param string $hosts[]['host']                             Host technical name (optional).
+	 * @param string $hosts[]['name']                             Host visible name (optional).
+	 * @param array  $hosts[]['groups']                           An array of host group objects with IDs that host will be replaced to.
+	 * @param int    $hosts[]['status']                           Status of the host (optional).
+	 * @param array  $hosts[]['interfaces']                       An array of host interfaces data to be replaced.
+	 * @param int    $hosts[]['interfaces']['type']               Interface type.
+	 * @param int    $hosts[]['interfaces']['main']               Is this the default interface to use.
+	 * @param string $hosts[]['interfaces']['ip']                 Interface IP (optional).
+	 * @param int    $hosts[]['interfaces']['port']               Interface port (optional).
+	 * @param int    $hosts[]['interfaces']['useip']              Interface shoud use IP (optional).
+	 * @param string $hosts[]['interfaces']['dns']                Interface shoud use DNS (optional).
+	 * @param int    $hosts[]['interfaces']['bulk']               Use bulk requests for interface (optional).
+	 * @param int    $hosts[]['proxy_hostid']                     ID of the proxy that is used to monitor the host (optional).
+	 * @param int    $hosts[]['ipmi_authtype']                    IPMI authentication type (optional).
+	 * @param int    $hosts[]['ipmi_privilege']                   IPMI privilege (optional).
+	 * @param string $hosts[]['ipmi_username']                    IPMI username (optional).
+	 * @param string $hosts[]['ipmi_password']                    IPMI password (optional).
+	 * @param array  $hosts[]['tags']                             An array of tags (optional).
+	 * @param string $hosts[]['tags'][]['tag']                    Tag name.
+	 * @param string $hosts[]['tags'][]['value']                  Tag value.
+	 * @param array  $hosts[]['inventory']                        An array of host inventory data (optional).
+	 * @param array  $hosts[]['macros']                           An array of host macros (optional).
+	 * @param string $hosts[]['macros'][]['macro']                Host macro (required if "macros" is set).
+	 * @param array  $hosts[]['templates']                        An array of template objects with IDs that will be linked to host (optional).
+	 * @param string $hosts[]['templates'][]['templateid']        Template ID (required if "templates" is set).
+	 * @param array  $hosts[]['templates_clear']                  Templates to unlink and clear from the host (optional).
+	 * @param string $hosts[]['templates_clear'][]['templateid']  Template ID (required if "templates" is set).
+	 * @param string $hosts[]['tls_connect']                      Connections to host (optional).
+	 * @param string $hosts[]['tls_accept']                       Connections from host (optional).
+	 * @param string $hosts[]['tls_psk_identity']                 PSK identity (required if "PSK" type is set).
+	 * @param string $hosts[]['tls_psk']                          PSK (required if "PSK" type is set).
+	 * @param string $hosts[]['tls_issuer']                       Certificate issuer (optional).
+	 * @param string $hosts[]['tls_subject']                      Certificate subject (optional).
 	 *
 	 * @return array
 	 */
@@ -675,6 +703,8 @@ class CHost extends CHostGeneral {
 			}
 		}
 
+		$this->updateTags($hosts, 'hostid');
+
 		return ['hostids' => $hostids];
 	}
 
@@ -1156,6 +1186,42 @@ class CHost extends CHostGeneral {
 		if (!$nopermissions) {
 			$this->checkPermissions($hostIds, _('No permissions to referred object or it does not exist!'));
 		}
+
+		$this->validateDeleteCheckMaintenances($hostIds);
+	}
+
+	/**
+	 * Validates if hosts may be deleted, due to maintenance constrain.
+	 *
+	 * @throws APIException if a constrain failed
+	 *
+	 * @param array $hostids
+	 */
+	protected function validateDeleteCheckMaintenances(array $hostids) {
+		$maintenance = DBfetch(DBselect(
+			'SELECT m.name'.
+			' FROM maintenances m'.
+			' WHERE NOT EXISTS ('.
+				'SELECT NULL'.
+				' FROM maintenances_hosts mh'.
+				' WHERE m.maintenanceid=mh.maintenanceid'.
+					' AND '.dbConditionInt('mh.hostid', $hostids, true).
+			')'.
+				' AND NOT EXISTS ('.
+					'SELECT NULL'.
+					' FROM maintenances_groups mg'.
+					' WHERE m.maintenanceid=mg.maintenanceid'.
+				')'
+		));
+
+		if ($maintenance) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _n(
+				'Cannot delete host because maintenance "%1$s" must contain at least one host or host group.',
+				'Cannot delete selected hosts because maintenance "%1$s" must contain at least one host or host group.',
+				$maintenance['name'],
+				count($hostids)
+			));
+		}
 	}
 
 	/**
@@ -1312,19 +1378,40 @@ class CHost extends CHostGeneral {
 		return ['hostids' => $hostIds];
 	}
 
+	/**
+	 * Retrieves and adds additional requested data to the result set.
+	 *
+	 * @param array  $options
+	 * @param array  $result
+	 *
+	 * @return array
+	 */
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
 		$hostids = array_keys($result);
 
-		// adding inventories
+		// adding inventory
 		if ($options['selectInventory'] !== null) {
-			$relationMap = $this->createRelationMap($result, 'hostid', 'hostid');
 			$inventory = API::getApiService()->select('host_inventory', [
 				'output' => $options['selectInventory'],
-				'filter' => ['hostid' => $hostids]
+				'filter' => ['hostid' => $hostids],
+				'preservekeys' => true
 			]);
-			$result = $relationMap->mapOne($result, zbx_toHash($inventory, 'hostid'), 'inventory');
+
+			foreach ($hostids as $hostid) {
+				// There is no DB record if inventory mode is HOST_INVENTORY_DISABLED.
+				if (!array_key_exists($hostid, $inventory)) {
+					$inventory[$hostid] = [
+						'hostid' => (string) $hostid,
+						'inventory_mode' => (string) HOST_INVENTORY_DISABLED
+					];
+				}
+			}
+
+			$relation_map = $this->createRelationMap($result, 'hostid', 'hostid');
+			$inventory = $this->unsetExtraFields($inventory, ['hostid', 'inventory_mode'], $options['selectInventory']);
+			$result = $relation_map->mapOne($result, $inventory, 'inventory');
 		}
 
 		// adding hostinterfaces
@@ -1610,6 +1697,11 @@ class CHost extends CHostGeneral {
 			}
 
 			$groupids = array_merge($groupids, zbx_objectValues($host['groups'], 'groupid'));
+
+			// Validate tags.
+			if (array_key_exists('tags', $host)) {
+				$this->validateTags($host);
+			}
 		}
 		unset($host);
 
@@ -1905,6 +1997,11 @@ class CHost extends CHostGeneral {
 					}
 				}
 			}
+
+			// Validate tags.
+			if (array_key_exists('tags', $host)) {
+				$this->validateTags($host);
+			}
 		}
 		unset($host);
 
