@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -179,6 +179,13 @@ class CImage extends CApiService {
 			// validate image (size and format)
 			$this->checkImage($image['image']);
 
+			list(,, $img_type) = getimagesizefromstring($image['image']);
+
+			if (!in_array($img_type, [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG])) {
+				// Converting to PNG all images except PNG, JPEG and GIF
+				$image['image'] = $this->convertToPng($image['image']);
+			}
+
 			$imageid = get_dbid('images', 'imageid');
 			$values = [
 				'imageid' => $imageid,
@@ -278,6 +285,13 @@ class CImage extends CApiService {
 				// validate image
 				$this->checkImage($image['image']);
 
+				list(,, $img_type) = getimagesizefromstring($image['image']);
+
+				if (!in_array($img_type, [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG])) {
+					// Converting to PNG all images except PNG, JPEG and GIF
+					$image['image'] = $this->convertToPng($image['image']);
+				}
+
 				switch ($DB['TYPE']) {
 					case ZBX_DB_POSTGRESQL:
 						$values['image'] = "'".pg_escape_bytea($image['image'])."'";
@@ -579,4 +593,22 @@ class CImage extends CApiService {
 
 		return $sqlParts;
 	}
+
+	/**
+	 * Convert image body to PNG.
+	 *
+	 * @param string $image  Base64 encoded body of image.
+	 * @return string
+	 */
+	protected function convertToPng($image) {
+		$image = imagecreatefromstring($image);
+
+		ob_start();
+		imagealphablending($image, false);
+		imagesavealpha($image, true);
+		imagepng($image);
+		imagedestroy($image);
+
+		return ob_get_clean();
+	}
 }
