@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -282,7 +282,7 @@ class CPageFilter {
 	 * Retrieve objects stored in the user profile.
 	 * If the 'select_latest' option is used, the IDs will be loaded from the web.latest.objectid profile values,
 	 * otherwise - from the web.*.objectid field, depending on the use of the 'individual' option.
-	 * If the 'DDReset' option is used, IDs will be reset to zeroes.
+	 * If the 'DDReset' option is used, IDs will be reset to zeros.
 	 * The method also sets the scope for remembering the selected values, see the 'individual' option for more info.
 	 *
 	 * @param array $options
@@ -378,10 +378,11 @@ class CPageFilter {
 	 * @param array  $groups
 	 * @param string $groups[<groupid>]['groupid']
 	 * @param string $groups[<groupid>]['name']
+	 * @param array  $options                        HostGroup API call parameters.
 	 *
 	 * @return array
 	 */
-	public static function enrichParentGroups(array $groups) {
+	public static function enrichParentGroups(array $groups, array $options = []) {
 		$parents = [];
 		foreach ($groups as $group) {
 			$parent = explode('/', $group['name']);
@@ -399,11 +400,18 @@ class CPageFilter {
 		}
 
 		if ($parents) {
-			$groups += API::HostGroup()->get([
-				'output' => ['groupid', 'name'],
-				'filter' => ['name' => array_keys($parents)],
-				'preservekeys' => true
-			]);
+			if (!array_key_exists('output', $options)) {
+				$options['output'] = ['groupid', 'name'];
+			}
+
+			if (!array_key_exists('filter', $options)) {
+				$options['filter'] = [];
+			}
+
+			$options['filter']['name'] = array_keys($parents);
+
+			$options['preservekeys'] = true;
+			$groups += API::HostGroup()->get($options);
 		}
 
 		return $groups;
@@ -429,7 +437,7 @@ class CPageFilter {
 
 		CArrayHelper::sort($this->data['groups'], ['name']);
 
-		// select remembered selection
+		// Select remembered selection.
 		if ($groupId === null && $this->_profileIds['groupid'] > 0) {
 			// set group only if host is in group or hostid is not set
 			$host = null;
@@ -437,8 +445,18 @@ class CPageFilter {
 
 			if ($hostId) {
 				// Get child groups for "profileIds['groupid']".
-				$profileids = [$this->_profileIds['groupid']];
-				$parent = $this->data['groups'][$this->_profileIds['groupid']]['name'].'/';
+				$group = reset($this->data['groups']);
+				$profileids = [];
+
+				if (array_key_exists($this->_profileIds['groupid'], $this->data['groups'])) {
+					$profileids = [$this->_profileIds['groupid']];
+					$parent = $this->data['groups'][$this->_profileIds['groupid']]['name'].'/';
+				}
+				elseif (array_key_exists($group['groupid'], $this->data['groups'])) {
+					$profileids = [$group['groupid']];
+					$parent = $this->data['groups'][$group['groupid']]['name'].'/';
+				}
+
 				foreach ($this->data['groups'] as $group) {
 					if (strpos($group['name'], $parent) === 0) {
 						$profileids[] = $group['groupid'];
@@ -573,8 +591,8 @@ class CPageFilter {
 				}
 			}
 
-			// select remembered selection
-			if ($hostId === null) {
+			// Select stored hostid, ignore zero value ('All hosts').
+			if ($hostId === null && $this->_profileIds['hostid'] > 0) {
 				$hostId = $this->_profileIds['hostid'];
 			}
 
@@ -709,7 +727,7 @@ class CPageFilter {
 	/**
 	 * Initialize minimum trigger severities.
 	 *
-	 * @param string $severityMin			minimum trigger severity
+	 * @param string $severityMin			minimum severity
 	 * @param array  $options				array of options
 	 * @param int    $options['default']	default severity
 	 * @param string $options['mapId']		ID of a map
@@ -751,30 +769,43 @@ class CPageFilter {
 	/**
 	 * Get hosts combobox with selected item.
 	 *
+	 * @param array  $options
+	 * @param array  $options['action']		onChange action.
+	 *
 	 * @return CComboBox
 	 */
-	public function getHostsCB() {
+	public function getHostsCB(array $options = []) {
 		$items = $classes = [];
 		foreach ($this->hosts as $id => $host) {
 			$items[$id] = $host['name'];
 			$classes[$id] = ($host['status'] == HOST_STATUS_NOT_MONITORED) ? ZBX_STYLE_RED : null;
 		}
-		$options = ['objectName' => 'hosts', 'classes' => $classes];
+		$options += ['objectName' => 'hosts', 'classes' => $classes];
 
-		return $this->_getCB('hostid', $this->hostid, $items, $options);
+		return $this->_getCB('hostid', $this->hostid, $items, $options)
+					->addStyle('max-width: '.ZBX_TEXTAREA_FILTER_STANDARD_WIDTH.'px;');
 	}
 
 	/**
 	 * Get host groups combobox with selected item.
 	 *
+	 * @param array  $options
+	 * @param array  $options['action']		onChange action.
+	 *
 	 * @return CComboBox
 	 */
-	public function getGroupsCB() {
+	public function getGroupsCB(array $options = []) {
 		$items = [];
 		foreach ($this->groups as $id => $group) {
 			$items[$id] = $group['name'];
 		}
-		return $this->_getCB('groupid', $this->groupid, $items, ['objectName' => 'groups']);
+
+		$options += [
+			'objectName' => 'groups'
+		];
+
+		return $this->_getCB('groupid', $this->groupid, $items, $options)
+					->addStyle('max-width: '.ZBX_TEXTAREA_FILTER_STANDARD_WIDTH.'px;');
 	}
 
 	/**
@@ -832,11 +863,16 @@ class CPageFilter {
 	 * @param array  $options
 	 * @param string $options['objectName']
 	 * @param array  $options['classes']	array of class names for the combobox options with item IDs as keys
+	 * @param array  $options['action']		onChange action. Default value is 'submit()'.
 	 *
 	 * @return CComboBox
 	 */
 	private function _getCB($name, $selectedId, $items, array $options = []) {
-		$comboBox = new CComboBox($name, $selectedId, 'javascript: submit();');
+		if (!array_key_exists('action', $options)) {
+			$options['action'] = 'javascript: submit();';
+		}
+
+		$comboBox = (new CComboBox($name, $selectedId, $options['action']))->removeId();
 
 		natcasesort($items);
 
