@@ -23,39 +23,6 @@
 		</td>
 	</tr>
 </script>
-<?php if (!$data['is_discovery_rule']) : ?>
-	<script type="text/x-jquery-tmpl" id="preprocessing_steps_row">
-	<?php
-		$preproc_types_cbbox = new CComboBox('preprocessing[#{rowNum}][type]', '');
-
-		foreach (get_preprocessing_types() as $group) {
-			$cb_group = new COptGroup($group['label']);
-
-			foreach ($group['types'] as $type => $label) {
-				$cb_group->addItem(new CComboItem($type, $label));
-			}
-
-			$preproc_types_cbbox->addItem($cb_group);
-		}
-
-		echo (new CRow([
-			$readonly
-				? null
-				: (new CCol(
-					(new CDiv())->addClass(ZBX_STYLE_DRAG_ICON)
-				))->addClass(ZBX_STYLE_TD_DRAG_ICON),
-				$preproc_types_cbbox,
-				(new CTextBox('preprocessing[#{rowNum}][params][0]', ''))->setAttribute('placeholder', _('pattern')),
-				(new CTextBox('preprocessing[#{rowNum}][params][1]', ''))->setAttribute('placeholder', _('output')),
-				(new CButton('preprocessing[#{rowNum}][remove]', _('Remove')))
-					->addClass(ZBX_STYLE_BTN_LINK)
-					->addClass('element-table-remove')
-		]))
-			->addClass('sortable')
-			->toString()
-	?>
-	</script>
-<?php endif ?>
 <script type="text/javascript">
 	jQuery(function($) {
 		$('#delayFlexTable').on('click', 'input[type="radio"]', function() {
@@ -76,97 +43,6 @@
 		$('#delayFlexTable').dynamicRows({
 			template: '#delayFlexRow'
 		});
-
-		<?php if (!$data['is_discovery_rule']) : ?>
-			var preproc_row_tpl = new Template($('#preprocessing_steps_row').html()),
-				preprocessing = $('#preprocessing');
-
-			preprocessing.sortable({
-				disabled: (preprocessing.find('tr.sortable').length < 2),
-				items: 'tr.sortable',
-				axis: 'y',
-				cursor: 'move',
-				containment: 'parent',
-				handle: 'div.<?= ZBX_STYLE_DRAG_ICON ?>',
-				tolerance: 'pointer',
-				opacity: 0.6,
-				helper: function(e, ui) {
-					ui.children().each(function() {
-						var td = $(this);
-
-						td.width(td.width());
-					});
-
-					return ui;
-				},
-				start: function(e, ui) {
-					$(ui.placeholder).height($(ui.helper).height());
-				}
-			});
-
-			preprocessing
-				.on('click', '.element-table-add', function() {
-					var row = $(this).parent().parent();
-					row.before(preproc_row_tpl.evaluate({rowNum: preprocessing.find('tr.sortable').length}));
-
-					if (preprocessing.find('tr.sortable').length > 1) {
-						preprocessing.sortable('enable');
-					}
-				})
-				.on('click', '.element-table-remove', function() {
-					var row = $(this).parent().parent();
-					row.remove();
-
-					if (preprocessing.find('tr.sortable').length < 2) {
-						preprocessing.sortable('disable');
-					}
-				})
-				.on('change', 'select[name*="type"]', function() {
-					var inputs = $(this).parent().parent().find('[name*="params"]');
-
-					switch ($(this).val()) {
-						case '<?= ZBX_PREPROC_MULTIPLIER ?>':
-							$(inputs[0])
-								.show()
-								.attr('placeholder', <?= CJs::encodeJson(_('number')) ?>);
-							$(inputs[1]).hide();
-							break;
-
-						case '<?= ZBX_PREPROC_RTRIM ?>':
-						case '<?= ZBX_PREPROC_LTRIM ?>':
-						case '<?= ZBX_PREPROC_TRIM ?>':
-							$(inputs[0])
-								.show()
-								.attr('placeholder', <?= CJs::encodeJson(_('list of characters')) ?>);
-							$(inputs[1]).hide();
-							break;
-
-						case '<?= ZBX_PREPROC_XPATH ?>':
-						case '<?= ZBX_PREPROC_JSONPATH ?>':
-							$(inputs[0])
-								.show()
-								.attr('placeholder', <?= CJs::encodeJson(_('path')) ?>);
-							$(inputs[1]).hide();
-							break;
-
-						case '<?= ZBX_PREPROC_REGSUB ?>':
-							$(inputs[0])
-								.show()
-								.attr('placeholder', <?= CJs::encodeJson(_('pattern')) ?>);
-							$(inputs[1]).show();
-							break;
-
-						case '<?= ZBX_PREPROC_BOOL2DEC ?>':
-						case '<?= ZBX_PREPROC_OCT2DEC ?>':
-						case '<?= ZBX_PREPROC_HEX2DEC ?>':
-						case '<?= ZBX_PREPROC_DELTA_VALUE ?>':
-						case '<?= ZBX_PREPROC_DELTA_SPEED ?>':
-							$(inputs[0]).hide();
-							$(inputs[1]).hide();
-							break;
-					}
-				});
-		<?php endif ?>
 	});
 </script>
 <?php
@@ -217,6 +93,8 @@ if (!empty($this->data['interfaces'])) {
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_JMX, 'interfaceid');
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SNMPTRAP, 'interface_row');
 	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SNMPTRAP, 'interfaceid');
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, 'interface_row');
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, 'interfaceid');
 }
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SIMPLE, 'row_username');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_SIMPLE, 'username');
@@ -281,6 +159,18 @@ zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_CALCULATED, 'row_para
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_TRAPPER, 'trapper_hosts');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_TRAPPER, 'row_trapper_hosts');
 zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_DEPENDENT, 'row_master_item');
+$ui_rows = [
+	ITEM_TYPE_HTTPAGENT => [
+		'url_row', 'query_fields_row', 'request_method_row', 'timeout_row', 'post_type_row', 'posts_row', 'headers_row',
+		'status_codes_row', 'follow_redirects_row', 'retrieve_mode_row', 'output_format_row', 'allow_traps_row',
+		'request_method', 'http_proxy_row', 'http_authtype_row', 'http_authtype', 'verify_peer_row', 'verify_host_row',
+		'ssl_key_file_row', 'ssl_cert_file_row', 'ssl_key_password_row', 'trapper_hosts', 'allow_traps'
+	]
+];
+foreach ($ui_rows[ITEM_TYPE_HTTPAGENT] as $row) {
+	zbx_subarray_push($this->data['typeVisibility'], ITEM_TYPE_HTTPAGENT, $row);
+}
+
 foreach ($this->data['types'] as $type => $label) {
 	switch ($type) {
 		case ITEM_TYPE_DB_MONITOR:
@@ -349,6 +239,7 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'p
 zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'row_publickey');
 zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'privatekey');
 zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'row_privatekey');
+
 ?>
 <script type="text/javascript">
 	function setAuthTypeLabel() {
@@ -361,7 +252,7 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'r
 		}
 	}
 
-	jQuery(document).ready(function() {
+	jQuery(document).ready(function($) {
 		<?php
 		if (!empty($this->data['authTypeVisibility'])) { ?>
 			var authTypeSwitcher = new CViewSwitcher('authtype', 'change',
@@ -371,24 +262,88 @@ zbx_subarray_push($this->data['authTypeVisibility'], ITEM_AUTHTYPE_PUBLICKEY, 'r
 			var typeSwitcher = new CViewSwitcher('type', 'change',
 				<?php echo zbx_jsvalue($this->data['typeVisibility'], true); ?>,
 				<?php echo zbx_jsvalue($this->data['typeDisable'], true); ?>);
-		<?php }
+		<?php } ?>
+		if ($('#http_authtype').length) {
+			new CViewSwitcher('http_authtype', 'change', <?= zbx_jsvalue([
+				HTTPTEST_AUTH_BASIC => ['http_username_row', 'http_password_row'],
+				HTTPTEST_AUTH_NTLM => ['http_username_row', 'http_password_row']
+			], true) ?>);
+		}
+		<?php
 		if (!empty($this->data['securityLevelVisibility'])) { ?>
 			var securityLevelSwitcher = new CViewSwitcher('snmpv3_securitylevel', 'change',
 				<?php echo zbx_jsvalue($this->data['securityLevelVisibility'], true); ?>);
 		<?php } ?>
 
-		jQuery('#type')
+		if ($('#allow_traps').length) {
+			new CViewSwitcher('allow_traps', 'change', <?= zbx_jsvalue([
+				HTTPCHECK_ALLOW_TRAPS_ON => ['row_trapper_hosts']
+			], true) ?>);
+		}
+
+		$('#type')
 			.change(function() {
 				// update the interface select with each item type change
 				var itemInterfaceTypes = <?php echo CJs::encodeJson(itemTypeInterface()); ?>;
-				organizeInterfaces(itemInterfaceTypes[parseInt(jQuery(this).val())]);
+				organizeInterfaces(itemInterfaceTypes[parseInt($(this).val())]);
 
 				setAuthTypeLabel();
 			})
 			.trigger('change');
 
-		jQuery('#authtype').bind('change', function() {
+		$('#authtype').bind('change', function() {
 			setAuthTypeLabel();
 		});
+
+		$('[data-action="parse_url"]').click(function() {
+			var url_node = $(this).siblings('[name="url"]'),
+				table = $('#query_fields_pairs').data('editableTable'),
+				url = parseUrlString(url_node.val())
+
+			if (typeof url === 'object') {
+				if (url.pairs.length > 0) {
+					table.addRows(url.pairs);
+					table.getTableRows().map(function() {
+						var empty = $(this).find('input[type="text"]').map(function() {
+							return $(this).val() == '' ? this : null;
+						});
+
+						return empty.length == 2 ? this : null;
+					}).map(function() {
+						table.removeRow(this);
+					});
+				}
+
+				url_node.val(url.url);
+			}
+			else {
+				overlayDialogue({
+					'title': <?= CJs::encodeJson(_('Error')); ?>,
+					'content': $('<span>').html(<?=
+						CJs::encodeJson(_('Failed to parse URL.').'<br><br>'._('URL is not properly encoded.'));
+					?>),
+					'buttons': [
+						{
+							title: <?= CJs::encodeJson(_('Ok')); ?>,
+							class: 'btn-alt',
+							focused: true,
+							action: function() {}
+						}
+					]
+				});
+			}
+		});
+
+		$('#request_method').change(function() {
+			if ($(this).val() == <?= HTTPCHECK_REQUEST_HEAD ?>) {
+				$(':radio', '#retrieve_mode')
+					.filter('[value=<?= HTTPTEST_STEP_RETRIEVE_MODE_HEADERS ?>]').click()
+					.end()
+					.attr('disabled', 'disabled');
+			}
+			else {
+				$(':radio', '#retrieve_mode').removeAttr('disabled');
+			}
+		});
 	});
 </script>
