@@ -20,6 +20,15 @@
 
 jQuery(function($) {
 
+	$.propHooks.disabled = {
+		set: function (el, val) {
+			if (el.disabled !== val) {
+				el.disabled = val;
+				$(el).trigger(val ? 'disable' : 'enable');
+			}
+		}
+	};
+
 	var $search = $('#search');
 
 	if ($search.length) {
@@ -61,6 +70,28 @@ jQuery(function($) {
 		changeClass(comboBox);
 	});
 
+	function uncheckedHandler($checkbox) {
+		var $hidden = $checkbox.prev('input[type=hidden][name="' + $checkbox.prop('name') + '"]');
+
+		if ($checkbox.is(':checked') || $checkbox.is(':disabled')) {
+			$hidden.remove();
+		}
+		else if (!$hidden.length) {
+			$('<input>', {'type': 'hidden', 'name': $checkbox.prop('name')})
+				.val($checkbox.attr('unchecked-value'))
+				.insertBefore($checkbox);
+		}
+	}
+
+	$('input[unchecked-value]').each(function() {
+		var $this = $(this);
+
+		uncheckedHandler($this);
+		$this.on('change enable disable', function() {
+			uncheckedHandler($(this));
+		});
+	});
+
 	function showMenuPopup(obj, data, event) {
 		switch (data.type) {
 			case 'history':
