@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -125,6 +125,7 @@ else {
 
 	$widget->setControls((new CForm('get'))
 		->cleanItems()
+		->setAttribute('aria-label', _('Main filter'))
 		->addItem($controls)
 	);
 
@@ -137,7 +138,10 @@ else {
 	]);
 
 	foreach ($db_users as $user_data) {
-		$header[] = (new CColHeader(getUserFullname($user_data)))->addClass('vertical_rotation');
+		$full_name = getUserFullname($user_data);
+		$header[] = (new CColHeader($full_name))
+			->addClass('vertical_rotation')
+			->setTitle($full_name);
 		$users[] = $user_data['userid'];
 	}
 
@@ -275,7 +279,9 @@ else {
 
 		$links = [];
 		foreach ($media_types as $id => $description) {
-			$links[] = new CLink($description, 'zabbix.php?action=mediatype.edit&mediatypeid='.$id);
+			$links[] = (CWebUser::getType() < USER_TYPE_SUPER_ADMIN)
+				? $description
+				: new CLink($description, 'zabbix.php?action=mediatype.edit&mediatypeid='.$id);
 			$links[] = SPACE.'/'.SPACE;
 		}
 		array_pop($links);
